#!perl

# Reformat TACLog output RSGB EDI format into acceptable ADIF format for 
# eQSL.cc upload.  
# Incorporate optional Comment field.
#
# Copyleft  2003, 2004, 2005 David A Gilligan. <g1ogy\@g1ogy.com>

#############################################################################
# This program is free software; you can redistribute it and/or modify it   
# under the terms of the GNU General Public License as published by the 
# Free Software Foundation; either version 2 of the License, or (at your    
# option) any later version. 
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
# Public License for more details. 
#   http://www.m1cro.org.uk/Members/G1OGY/GNU-GPL.html
#
#     In a word: If it breaks - you get to keep all the pieces.
#
#############################################################################


use Getopt::Long;

# headers
my $prog = "EDI2eQSL";
my $version = "Beta2 v 0.22";
my $datestring="[20050206]";
my $auth = "Author:Dave Gilligan <g1ogy\@g1ogy.com> ";
my $arights="Licenced under GNU GPL.";


GetOptions('c|comment=s', 'e|epoch', 's|stripheader', 'h|help');


# define Usage

    die "\n\"$prog\"  $version       $datestring     $auth
                                        $arights
  Usage:
  
	\"$prog\" [-c|comment \"<Comment Text>\"] [-h|help] \\ 
  		   [-s|stripheader] [-e|epoch] <Input-File> [<Output-File>]
  		
        \"$prog\" converts certain TACLog EDI file output to an 
        \'ADIF\' format but can include a freeform \'comment\'. 
            
        \"$prog\" requires an INPUT file name argument and 
        optionally, an OUTPUT file name.
        
        Crucially, if you are processing logs from the last millennium (19XX),
        it is essential to specify the \'-e\' flag to tell \"$prog\" that
        the logs are 20th Century.
        
        The input file MUST be that output from TACLog's EDI file 
        created in \"RSGB STD contest log data 1997\" format <MM 4-2-3>.    
        
        Please see the README file distributed with this program or
        execute  \"$prog\" -[h]elp to display the README here.

    Evidently, this program expects to be run under Linux or other 
    Un*x-style O/S or a 32-bit version of MS Windows.     \n"
    
    unless @ARGV;
    
    
    # Print Banner
	    unless ($opt_s)     {
	        Banner();
	    }

# define local variables

	my ($StaticComment) = $opt_c;
	my ($epochtime) = "";
    my ($TACfile)	=	$ARGV[0];
    my ($ADIFfile) 	= 	$ARGV[1];
    my ($OUTPUTf)	=	"";
    

# Sort the 19s from the dozens
    
    	if ($opt_e)	{
			$epochtime = 19;
		}
			else	 { 
				$epochtime=20;
			}

# Lookup tables (hashes)

our %Bandmap = (
	"50M"	=> "6M",
	"70M"	=> "4M",
	"144"   => "2M",
	"220"   => "1.25M",
	"432"   => "70CM",
	"902"   => "35CM",
	"1,3"   => "23CM",
	"2,3"   => "13CM",
	"3,4"   => "9CM",
	"5,7"   => "6CM",
	"10G"   => "3CM",
	"24G"   => "1.25CM",
	"47G"   => "6MM",
	"75G"   => "4MM",
	"120"   => "2.5MM",
	"142"   => "2MM",
	"241"   => "1MM"
  );  
    
  
	  
our %Modemap = (
	J3E   => "SSB",
	A1A   => "CW",
	F3E   => "FM"
  );
	
    
# initialise the logbook hash
our %Logbook = (
    qso_date    => '',
    time_on     => '',
    band        => '',
    mode        => '',
    call        => '',
    rst_sent    => '',
    stx         => '',
    rst_rcvd    => '',
    srx         => '',
    qrb         => '',
    operator    => '',
    gridsquare  => '',
    qdxcc       => '',
    qsl_comment     => ''
  );


###   
# Debug    
#
#my @print1 = %Bandmap;
#my @print2 = %Modemap;
#my @print3 = %Logbook;
#
#print "@print1\n";
#print "@print2\n";
#print "@print3\n";
#
#
#   print "$TACfile \n";
#   print "$StaticComment \n";
#
###
   
# main  

    if ($ARGV[1] ne "")  {
        open (ADIFOUT,">$ARGV[1]") || die "\nCannot Create Output file: \"$ARGV[1]\"!\n";
        $OUTPUTf	= $ARGV[1];
        }
        else  { 
            open (ADIFOUT,">$TACfile.ADIF") || die "\nCannot Create Output file: \"$TACfile.ADIF\"!\n"; 
            $OUTPUTf	= "$TACfile.ADIF";
            }
    open (EDIIN, "$TACfile") || die "\nInput File - \"$TACFile\" - not Found\! Check the path : \n$!";

   
    while (<EDIIN>) {

        chomp;
        
        # get each element of the input line into the Logbook hash
        	# this does not work for '70 M' or '50 M' (Thanks Bo!)
        	# need to write a pre-processor that replaces the default seperator
        	# and then uses that token as the arg to \'split\'
        	# Meanwhile block-edit these band EDI logs to remove the space
        	# between 70/50 and M
        	
        	@Logbook {
			    qso_date,    
			    time_on,     
			    band,        
			    mode,        
			    call,        
			    rst_sent,    
			    stx,         
			    rst_rcvd,    
			    srx,         
			    qrb,         
			    operator,    
			    gridsquare,  
			    qdxcc,       
			    qsl_comment  } = (split);
			    



	
#  Is it worth reducing the output to match the required input to EQSL.cc ??

 
 # Now process the array/hash

		Expanddate(); 
  		Addcomment();
  		Band_translate();
  		Mode_translate();
  		Reportconcat();

# and write the output file  			
	Output() ;

}

print "\n 	Output in $OUTPUTf\n\n";

	Credits();


   ############################################################################################
sub Expanddate {
	# Add in the century
				
		$Logbook{qso_date} = $epochtime . @Logbook{qso_date};
	}			
  
   ############################################################################################

sub Addcomment	{
	#Replace the (empty) comment value with the text from the command line
	
  		if ($opt_c)  {
  			$Logbook{qsl_comment} = $StaticComment;	
  			}
	}

   ############################################################################################
  
sub Band_translate	{
	# get value of band from hash - compare with Band's hash keys and replace the current value
	# with the value from the Band hash

	$Logbook{band} = $Bandmap{$Logbook{band}};
	}

   ############################################################################################
   
sub Mode_translate	{
	# As above - but mode
	
	$Logbook{mode} =  $Modemap{$Logbook{mode}} ;    
	}

   ############################################################################################

sub Reportconcat  {
	# concat the Sent report with the sent serial - 59/456

	$Logbook{rst_sent} = $Logbook{rst_sent} ."/" . $Logbook{stx} ;
	}

   ############################################################################################

sub elem {
	# Format the ADIF tags and data-count  attr. OK1FOU
	
    my ($tag, $data) = @_ ;
    return sprintf( '<%s:%d>%s', $tag, length($data), $data) ;
}

    ############################################################################################

sub Output {
	# Build the output string  - concatenating element by element by reference to the 'elim'
	# function
	
    my $out ;
    for (keys %Logbook)  { $out .= uc elem($_, $Logbook{$_}) }
    
    	print ADIFOUT $out.'<eor>',"\n";

###
#
# Debug
#
#
#  print $out;
#
###

}

   
    ############################################################################################
    # Banner
    
    sub Banner  {
        print ("\n\"$prog\"  $version  $datestring  $auth\n   $arights\n");
        print ("\n  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n");
        }
    
    ############################################################################################
    # Credits
    
    sub Credits {
        
        print ("    Credits: ");
        print ("\n      \"$prog\" is written against \"ActivePerl v5.6\" and compiled \n");
        print ("        - where relevant - with \"Perl2exe\" by Indigostar Software.\n\n"); 
        print ("        \"$prog\": GNU General Public Licence applies.\n");
        print ("\n");
        print ("        **Thanks are due to Jindra Vavruska (OK1FOU) for the basis \n" );
        print ("        of the idea and the `elim` subroutine reused here.**\n");
        print ("\n  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n");
        }
    
    ############################################################################################

#endofperl
    
